#!/bin/bash
# Patch to enable Continuity, NightShift & H.264 acceleration in cMP5,1 with upgraded Broadcom board
# HOW TO USE:
# 1. Drag (or Copy/past) '_cMP51_PATCH_' folder to the desktop --- DO NOT RENAME IT
# or unzip '_cMP51_PATCH_.zip' directly on desktop
# 2. Open Terminal
# 3. Type 'sudo'
# 4. Drag 'cmp51.sh' file into the Terminal
# 5. Hit 'Enter'
#
MOUNT="$HOME/mount_dir"
PAYLOADS="$HOME/Desktop/_cMP51_PATCH_/payloads"
PREBOOT_VOL=`diskutil info / | grep "APFS Volume Group" | awk '{print $4}'`
DISK_LABEL=`diskutil info / | grep "Volume Name:" | awk '{print $3,$4}'`
DISK_ID_FULL=`diskutil info / | grep "Device Identifier" | awk '{print $3}'`
DISK_ID=${DISK_ID_FULL::${#DISK_ID_FULL}-2}
BOARD_ID=`/usr/sbin/ioreg -l | grep board-id | awk -F\" '{ print $4 }' | grep Mac`
MODEL=`system_profiler SPHardwareDataType | grep "Model Identifier" | awk '{print $3}'`
OS_NAME=${1:-$(sw_vers -productName)}
OS_VER=${1:-$(sw_vers -productVersion)}
OS_BUILD=${1:-$(sw_vers -buildVersion)}
#
clear
echo "——— Fix Continuity, NightShift & H.264 hwaccel ———"
echo
echo "———  for cMP5,1 with upgraded Broadcom board   ———"
echo
echo "———  Copyright (c) 2021  INFORMATIQUE SERVICE  ———"
echo
#
# Checking if patch folder is on desktop
#
if [ ! -d $HOME/Desktop/_cMP51_PATCH_ ]
then
    echo
    echo -e "\033[0;31m!!!  '_cMP51_PATCH_' folder does not exist on Desktop (or has been renamed)  !!!\033[0m"
    echo
    exit 1
fi
#
# Check if volume have macOS Big Sur (build 20...)
#
if [[ "$OS_BUILD" == 20* ]]
then
    echo -n ""
else
    echo
    echo -e "\033[0;31mERROR: Volume appears to have $OS_NAME $OS_VER ($OS_BUILD)\033[0m"
    echo -e "\033[0;31mPlease make sure you booted on macOS Big Sur volume\033[0m"
    echo
    exit 1
fi
#
# Checking System Integrity Protection status
#
if  csrutil status | grep 'disabled' &> /dev/null
then
    printf "SIP status:      \033[0;32mdisabled\033[0m\n"
    else
    printf "SIP status:      \033[0;31menabled\033[0m\n"
    echo
    echo -e "\033[0;31m!!! Error: SIP must be disabled !!!\033[0m"
    exit 1
    fi
if  csrutil authenticated-root status | grep 'disabled' &> /dev/null
then
    printf "SSV status:      \033[0;32mdisabled\033[0m\n"
    else
    printf "SSV status:      \033[0;31menabled\033[0m\n"
    echo
    echo -e "\033[0;31m!!! Error: SSV must be disabled !!!\033[0m"
    exit 1
    fi
echo
# Show system informations
#
sleep 0.5
echo -e "Product Name:    \033[0;036m$OS_NAME\033[0m"
sleep 0.5
echo -e "Product Version: \033[0;036m$OS_VER\033[0m"
sleep 0.5
echo -e "Build Version:   \033[0;036m$OS_BUILD\033[0m"
sleep 0.5
echo -e "Mac Model:       \033[0;036m$MODEL\033[0m"
sleep 0.5
echo -e "Board ID:        \033[0;036m$BOARD_ID\033[0m"
sleep 0.5
echo -e "Disk label:      \033[0;036m$DISK_LABEL\033[0m"
sleep 0.5
echo -e "Disk ID:         \033[0;036m$DISK_ID\033[0m"
sleep 0.5
#
# Create "mount_dir" folder in Home folder
#
if [ ! -d $HOME/mount_dir ]
then
    sudo mkdir $HOME/mount_dir
    sudo chown -R $(whoami):staff $HOME/mount_dir
fi
#
echo
echo
echo -n "1. Mounting Snapshot in $MOUNT for RW access."
sleep 0.1; echo -n "."; sleep 0.1; echo -n "."; sleep 0.1; echo -n "."; sleep 0.1; echo -n "."; sleep 0.1; echo -n "."; sleep 0.1; echo -n "."; sleep 0.1; echo "."
sudo mount -o nobrowse -t apfs /dev/$DISK_ID $MOUNT
if (($?))
then
    echo
    echo -e "\033[0;31m!!! Error: could not mount Snapshot $DISK_ID in $MOUNT for RW !!!\033[0m"
    echo
    echo -e "CHECK THE PATH '\033[0;032m$MOUNT\033[0m' IN THIS SCRIPT (USERNAME MAY NOT BE CORRECT)"
    echo
    exit 1
fi  
sleep 0.5
echo
echo -n "2. Patching IOBlueTooth.plist for cMP5,1 with upgraded Broadcom card."
sleep 0.1; echo -n "."; sleep 0.1; echo -n "."; sleep 0.1; echo -n "."; sleep 0.1; echo -n "."; sleep 0.1; echo -n "."; sleep 0.1; echo -n "."; sleep 0.1; echo "."
sudo /usr/libexec/PlistBuddy -c "Set:$BOARD_ID:ContinuitySupport true" \
    "$MOUNT/System/Library/Frameworks/IOBluetooth.framework/Versions/A/Resources/SystemParameters.plist"
if (($?))
then
    echo -e "\033[0;31m!!! Error: could not set Continuity to true !!!\033[0m"
    exit 1
fi
#
sleep 0.5
echo -n "3. Patching CoreBrightness to enable NightShift."
sleep 0.1; echo -n "."; sleep 0.1; echo -n "."; sleep 0.1; echo -n "."; sleep 0.1; echo -n "."; sleep 0.1; echo -n "."; sleep 0.1; echo -n "."; sleep 0.1; echo "."
sudo cp -rf $PAYLOADS/CoreBrightness.framework $MOUNT/System/Library/PrivateFrameworks/
sudo chmod 755 $MOUNT/System/Library/PrivateFrameworks/CoreBrightness.framework
sudo chown 0:0 $MOUNT/System/Library/PrivateFrameworks/CoreBrightness.framework  
#
sleep 0.5
echo -n "4. Disabling Library Validation."
sleep 0.1; echo -n "."; sleep 0.1; echo -n "."; sleep 0.1; echo -n "."; sleep 0.1; echo -n "."; sleep 0.1; echo -n "."; sleep 0.1; echo -n "."; sleep 0.1; echo "."
sudo defaults write /Library/Preferences/com.apple.security.libraryvalidation.plist DisableLibraryValidation -bool true
if (($?))
then
    echo -e "\033[0;31m!!! Error: could not Disabling Library Validation !!!\033[0m"
    exit 1
fi
#
sleep 0.5
echo -n "5. Adding cMP4,1 & cMP5,1 to PlatformSupport.plist."
sleep 0.1; echo -n "."; sleep 0.1; echo -n "."; sleep 0.1; echo -n "."; sleep 0.1; echo -n "."; sleep 0.1; echo -n "."; sleep 0.1; echo -n "."; sleep 0.1; echo "."
sudo /usr/libexec/PlistBuddy -c "Add:SupportedBoardIds: string Mac-F221BEC8" "$MOUNT/System/Library/CoreServices/PlatformSupport.plist"
sudo /usr/libexec/PlistBuddy -c "Add:SupportedModelProperties: string MacPro4,1" "$MOUNT/System/Library/CoreServices/PlatformSupport.plist"
sudo /usr/libexec/PlistBuddy -c "Add:SupportedModelProperties: string MacPro5,1" "$MOUNT/System/Library/CoreServices/PlatformSupport.plist"
if (($?))
then
    echo -e "\033[0;31m!!! Error: Add cMP4,1 & cMP5,1 to PlatformSupport.plist !!!\033[0m"
    echo
    #exit 1
fi
#
sleep 0.5
echo -n "6. Adding cMP4,1 & cMP5,1 to PlatformSupport.plist on PREBOOT Volume."
sleep 0.1; echo -n "."; sleep 0.1; echo -n "."; sleep 0.1; echo -n "."; sleep 0.1; echo -n "."; sleep 0.1; echo -n "."; sleep 0.1; echo -n "."; sleep 0.1; echo "."
sudo /usr/libexec/PlistBuddy -c "Add:SupportedBoardIds: string Mac-F221BEC8" "/System/Volumes/Preboot/$PREBOOT_VOL/System/Library/CoreServices/PlatformSupport.plist"
sudo /usr/libexec/PlistBuddy -c "Add:SupportedModelProperties: string MacPro4,1" "/System/Volumes/Preboot/$PREBOOT_VOL/System/Library/CoreServices/PlatformSupport.plist"
sudo /usr/libexec/PlistBuddy -c "Add:SupportedModelProperties: string MacPro5,1" "/System/Volumes/Preboot/$PREBOOT_VOL/System/Library/CoreServices/PlatformSupport.plist"
if (($?))
then
    echo -e "\033[0;31m!!! Error: could not Add cMP4,1 & cMP5,1 to PlatformSupport.plist on PREBOOT Volume !!!\033[0m"
    echo
    #exit 1
fi
#
sleep 0.5
echo -n "7. Patching 'AppleGVA.framework' for H.264 Hardware acceleration."
sleep 0.1; echo -n "."; sleep 0.1; echo -n "."; sleep 0.1; echo -n "."; sleep 0.1; echo -n "."; sleep 0.1; echo -n "."; sleep 0.1; echo -n "."; sleep 0.1; echo "."
sudo /usr/libexec/PlistBuddy -c "Add :System\ Support:$BOARD_ID:allowFrameReordering bool false" "$MOUNT/System/Library/PrivateFrameworks/AppleGVA.framework/Versions/A/Resources/Info.plist"
sudo /usr/libexec/PlistBuddy -c "Add :System\ Support:$BOARD_ID:hwe bool true" "$MOUNT/System/Library/PrivateFrameworks/AppleGVA.framework/Versions/A/Resources/Info.plist"
sudo /usr/libexec/PlistBuddy -c "Add :System\ Support:$BOARD_ID:hwe420 bool true" "$MOUNT/System/Library/PrivateFrameworks/AppleGVA.framework/Versions/A/Resources/Info.plist"
sudo /usr/libexec/PlistBuddy -c "Add :System\ Support:$BOARD_ID:interlacedOff bool true" "$MOUNT/System/Library/PrivateFrameworks/AppleGVA.framework/Versions/A/Resources/Info.plist"
sudo /usr/libexec/PlistBuddy -c "Add :System\ Support:$BOARD_ID:ke1 bool true" "$MOUNT/System/Library/PrivateFrameworks/AppleGVA.framework/Versions/A/Resources/Info.plist"
sudo /usr/libexec/PlistBuddy -c "Add :System\ Support:$BOARD_ID:pp integer 1" "$MOUNT/System/Library/PrivateFrameworks/AppleGVA.framework/Versions/A/Resources/Info.plist"
if (($?))
then
    echo -e "\033[0;31m!!! Error: could not Patch AppleGVA.framework !!!\033[0m"
    echo
    #exit 1
fi
#
sleep 0.5
echo -n "8. Creating 'com.apple.AppleGVA' for H.264 Hardware acceleration."
sleep 0.1; echo -n "."; sleep 0.1; echo -n "."; sleep 0.1; echo -n "."; sleep 0.1; echo -n "."; sleep 0.1; echo -n "."; sleep 0.1; echo -n "."; sleep 0.1; echo "."
defaults write com.apple.AppleGVA gvaForceAMDKE -bool YES
defaults write com.apple.AppleGVA gvaForceAMDAVCEncode -bool YES
defaults write com.apple.AppleGVA gvaForceAMDAVCDecode -bool YES
defaults write com.apple.AppleGVA gvaForceAMDHEVCDecode -bool YES
#
sleep 0.5
echo -n "9. Creating new Snapshot."
sleep 0.1; echo -n "."; sleep 0.1; echo -n "."; sleep 0.1; echo -n "."; sleep 0.1; echo -n "."; sleep 0.1; echo -n "."; sleep 0.1; echo -n "."; sleep 0.1; echo "."
sudo bless --folder $MOUNT/System/Library/CoreServices --bootefi --create-snapshot
if (($?))
then
    echo -e "\033[0;31m!!! Error: could not bless snapshot with $MOUNT !!!\033[0m"
    exit 1
fi
#
sleep 1
echo
echo -e "\033[0;32m———      DONE !     ———\033[0m"
echo
echo -e "\033[0;31m∆∆∆\033[0m TO FINALIZE, Mac Pro MUST BE RESTARTED \033[0;31m∆∆∆\033[0m"
echo
read -p "If you wish to reboot type (Y): " -n 1 -r
echo
if [[ ! $REPLY =~ ^[Yy]$ ]]
then
    clear
    exit 1
fi
sudo shutdown -r now
exit 0